# Node.js Interop for Dart
![Dart](https://img.shields.io/badge/dart-%3E%3D1.24-brightgreen.svg) ![Node.js](https://img.shields.io/badge/node-%3E%3D8.9-brightgreen.svg) ![Release](https://img.shields.io/pub/v/nodejs_interop.svg) ![License](https://img.shields.io/badge/license-MIT-blue.svg) ![Coverage](https://coveralls.io/repos/github/cedx/nodejs-interop.dart/badge.svg) ![Build](https://travis-ci.org/cedx/nodejs-interop.dart.svg)

This package provides [Node.js](https://nodejs.org) interoperability for [Dart](https://www.dartlang.org).

## Requirements
- The latest [Dart SDK](https://www.dartlang.org) and [Pub](https://pub.dartlang.org) versions.
- The latest [Node.js](https://nodejs.org) and [npm](https://www.npmjs.com) versions.

If you plan to play with the sources, you will also need the latest [Grinder](http://google.github.io/grinder.dart) version.

## Installing via [Pub](https://pub.dartlang.org)

### 1. Depend on it
Add this to your package's `pubspec.yaml` file:

```yaml
dependencies:
  nodejs_interop: *
```

### 2. Install it
Install this package and its dependencies from a command prompt:

```shell
$ pub get
```

### 3. Import it
Now in your [Dart](https://www.dartlang.org) code, you can use:

```dart
// Dart abstractions for I/O support.
import 'package:nodejs_interop/io.dart';

// JavaScript and Node.js interoperability.
import 'package:nodejs_interop/js.dart' as js;
import 'package:nodejs_interop/node.dart' as node;
```

## Usage
- [I/O support](https://github.com/cedx/nodejs-interop.dart/blob/master/doc/io.md): file, socket, HTTP, and other I/O support for Node.js applications.
- [JavaScript](https://github.com/cedx/nodejs-interop.dart/blob/master/doc/js.md): support for interoperating with JavaScript APIs.
- [Node.js](https://github.com/cedx/nodejs-interop.dart/blob/master/doc/node.md): support for interoperating with Node.js APIs.
- [npm packages](https://github.com/cedx/nodejs-interop.dart/blob/master/doc/npm.md): support for interoperating with a predefined set of `npm` packages.
- [Pub transformer](https://github.com/cedx/nodejs-interop.dart/blob/master/doc/pub.md): automate the addition of a Node.js preamble to build assets.

## Examples
The following projects use this library to provide cross-platform features:

- [Coveralls for Dart](https://github.com/cedx/coveralls.dart): send LCOV and Clover coverage reports to the Coveralls service.
- [Reverse-Proxy.dart](https://github.com/cedx/reverse-proxy.dart): simple reverse proxy server supporting WebSockets.
- [Where.dart](https://github.com/cedx/where.dart): find the instances of an executable in the system path.

## Unit tests
In order to run the tests, you must install the [Node.js](https://nodejs.org) test dependencies by running the following command:

```shell
$ npm install
```

Then, you can launch the test runner from the command prompt:

```shell
$ pub run test
```

## See also
- [API reference](https://cedx.github.io/nodejs-interop.dart)
- [Code coverage](https://coveralls.io/github/cedx/nodejs-interop.dart)
- [Continuous integration](https://travis-ci.org/cedx/nodejs-interop.dart)

## License
[Node.js Interop for Dart](https://github.com/cedx/nodejs-interop.dart) is distributed under the MIT License.
