# npm packages
The `nodejs_interop.npm` library provides interoperability with a predefined set of [npm](https://www.npmjs.com) packages. They are used internally to provide features not natively supported by [Node.js](https://nodejs.org).

```dart
import 'package:nodejs_interop/npm.dart';
```

## The [`del`](https://www.npmjs.com/package/del) package
This package is used to recursively delete a directory and all its sub-directories and files:

```dart
import 'package:nodejs_interop/js.dart' show asFuture;
import 'package:nodejs_interop/node.dart' show require;

DelModule del = require('del');
var files = await asFuture(del('path/to/**/*.dart', new DelOptions(dryRun: true)));
print(files); // ["path/to/file1.dart", "path/to/subdir1/file2.dart", ...]
```

## The [`ipaddr.js`](https://www.npmjs.com/package/ipaddr.js) package
This package is used to parse and validate Internet addresses:

```dart
import 'package:nodejs_interop/node.dart' show require;

var address = '::ffff:163.172.221.182';
IPAddrModule ipaddr = require('ipaddr.js');
print(ipaddr.isValid(address)); // true
print(ipaddr.parse(address).toString()); // "::ffff:a3ac:ddb6"
print(ipaddr.process(address).toString()); // "163.172.221.182"
```

## The [`make-dir`](https://www.npmjs.com/package/make-dir) package
This package is used to recursively create all non-existing path components of a file system entity:

```dart
import 'package:nodejs_interop/js.dart' show asFuture;
import 'package:nodejs_interop/node.dart' show require;

MakeDirModule makeDir = require('make-dir');
var perms = int.parse('755', radix: 8);
await asFuture(makeDir('path/to/dir', new MakeDirOptions(mode: perms)));
```

## The [`os-locale`](https://www.npmjs.com/package/os-locale) package
This package is used to get the name of the current locale:

```dart
import 'package:nodejs_interop/js.dart' show asFuture;
import 'package:nodejs_interop/node.dart' show require;

OSLocaleModule osLocale = require('os-locale');
print(await asFuture(osLocale(new OSLocaleOptions(spawn: false)))); // "en_US"
```

## The [`readdir-enhanced`](https://www.npmjs.com/package/readdir-enhanced) package
This package is used to recursively list the sub-directories and files of a directory:

```dart
import 'package:nodejs_interop/js.dart' show asFuture;
import 'package:nodejs_interop/node.dart' show require;

ReaddirModule readdir = require('readdir-enhanced');
var files = await asFuture(readdir('path/to/dir', new ReaddirOptions(deep: true)));
print(files); // ["subdir1", "subdir1/file.txt", "subdir1/subdir2", ...]
```

## The [`supports-color`](https://www.npmjs.com/package/supports-color) package
This package is used to determine if ANSI codes are supported when `stdout` is connected to a terminal:

```dart
import 'package:nodejs_interop/node.dart' show require;

SupportsColorModule supportsColor = require('supports-color');
print(supportsColor.stderr); // false
print(supportsColor.stdout.has256); // true
```
