# Pub transformer
[Node.js](https://nodejs.org) applications built from [Dart](https://www.dartlang.org) code needs a special [preamble](https://pub.dartlang.org/packages/node_preamble) in order to be run from the command line.

The `nodejs_interop.pub` library provides a [Pub](https://pub.dartlang.org) transformer that ease the process of adding the preamble to the build assets.

To use the transformer, add this to your package's `pubspec.yaml` file:

```yaml
transformers:
- $dart2js
- nodejs_interop
```

> The transformer provided by this library must always be placed **after** the [dart2js transformer](https://webdev.dartlang.org/tools/pub/dart2js-transformer).

## Options

### `bool minify`
A value indicating whether to minify the Node.js [preamble](https://pub.dartlang.org/packages/node_preamble).
Defaults to `false` when `pub build` is run in `debug` mode, defaults to `true` in `release` mode.

You can force this feature (or completely disable it) by setting the `minify` option:

```yaml
transformers:
- $dart2js
- nodejs_interop:
    minify: true
```

### `bool shebang = false`
A value indicating whether to add a Node.js [shebang](https://en.wikipedia.org/wiki/Shebang_(Unix)) (i.e. `#!/usr/bin/env node`) at the beginning of the generated assets:

```yaml
transformers:
- $dart2js
- nodejs_interop:
    shebang: true
```

> The resulting JavaScript files are not executable, even with the shebang.
> On POSIX systems, you must `chmod +x` these files in order to set them executable.
