part of nodejs_interop.js;

/// A generic JavaScript error.
@JS('Error')
class JSError implements Exception {

  /// Creates a new error.
  external JSError([String message]);

  /// The number of stack frames collected by a stack trace. Defaults to `10`.
  external static int get stackTraceLimit;
  external static set stackTraceLimit(int value);

  /// A label that identifies the kind of error.
  external String get code;
  external set code(String value);

  /// The description of the error.
  external String get message;
  external set message(String value);

  /// A name for the type of error. Defaults to `"Error"`.
  external String get name;
  external set name(String value);

  /// A string describing the point in the code at which the error was instantiated.
  external String get stack;

  /// Creates a [stack] property on [targetObject],
  /// which when accessed returns a string representing the location in the code at which [captureStackTrace] was called.
  external void captureStackTrace(Object targetObject, [Function constructorOpt]);
}

/// Indicates that a program is not valid JavaScript.
/// These errors may only be generated and propagated as a result of code evaluation.
@JS('EvalError')
class JSEvalError extends JSError {

  /// Creates a new syntax error.
  external JSEvalError([String message]);
}

/// Indicates that a provided argument was not within the set or range of acceptable values for a function;
/// whether that is a numeric range, or outside the set of options for a given function parameter.
@JS('RangeError')
class JSRangeError extends JSError {

  /// Creates a new range error.
  external JSRangeError([String message]);
}

/// Indicates that an attempt is being made to access a variable that is not defined.
/// Such errors commonly indicate typos in code, or an otherwise broken program.
@JS('ReferenceError')
class JSReferenceError extends JSError {

  /// Creates a new reference error.
  external JSReferenceError([String message]);
}

/// Indicates that a program is not valid JavaScript.
/// These errors may only be generated and propagated as a result of code evaluation.
@JS('SyntaxError')
class JSSyntaxError extends JSError {

  /// Creates a new syntax error.
  external JSSyntaxError([String message]);
}

/// Indicates that a provided argument is not an allowable type.
/// For example, passing a function to a parameter which expects a string would be considered a [JSTypeError].
@JS('TypeError')
class JSTypeError extends JSError {

  /// Creates a new type error.
  external JSTypeError([String message]);
}

/// Indicates that a global URI handling function was used in a wrong way.
@JS('URIError')
class JSUriError extends JSError {

  /// Creates a new type error.
  external JSUriError([String message]);
}
