part of nodejs_interop.js;

/// Converts the specified [promise] to a [Future].
Future<T> asFuture<T>(Promise<T> promise) {
  var completer = new Completer<T>();
  promise.then(allowInterop(completer.complete), allowInterop(completer.completeError));
  return completer.future;
}

/// Converts the specified [future] to a [Promise].
Promise<T> asPromise<T>(Future<T> future) => new Promise<T>(allowInterop((resolve, reject) => future.then(resolve, onError: reject)));

/// Recursively converts the specified JavaScript [value] to a Dart object.
dynamic dartify(Object value) {
  if (value == null) return null;
  if (value is bool || value is num || value is DateTime || value is String || value is TypedData) return value;
  if (value is List) return value.map(dartify).toList();
  return new Map<String, dynamic>.fromIterable(_getProperties(value), key: (item) => item.first.toString(), value: (item) => dartify(item.last));
}

/// Removes a property from the specified JavaScript [object].
/// Returns a value indicating whether the property was successfully deleted.
@JS('Reflect.deleteProperty')
external bool deleteProperty(Object object, Object propertyKey);

/// Returns a map of the own enumerable properties of the specified JavaScript [object].
Map getProperties(Object object) => new Map.fromIterable(_getProperties(object), key: (item) => item.first, value: (item) => item.last);

/// Returns a list of the own enumerable property `[key, value]` pairs of the specified JavaScript [object].
@JS('Object.entries')
external List<List> _getProperties(Object object);
