part of nodejs_interop.node;

/// The [`assert`](https://nodejs.org/api/assert.html) module.
@JS()
abstract class AssertModule implements Function {

  /// An alias for the [ok] method.
  external void call(Object value, [String message]);

  /// Tests for deep equality between the [actual] and [expected] parameters.
  external void deepEqual(Object actual, Object expected, [String message]);

  /// Tests for deep strict equality between the [actual] and [expected] parameters.
  external void deepStrictEqual(Object actual, Object expected, [String message]);

  /// Asserts that the function [block] does not throw an [error].
  external void doesNotThrow(void block(), [Object error, String message]);

  /// Tests shallow, coercive equality between the [actual] and [expected] parameters using the abstract equality comparison (`==`).
  external void equal(Object actual, Object expected, [String message]);

  /// Throws an assertion error.
  external void fail([Object actualOrMessage, Object expected, String message, String operator, Function stackStartFunction]);

  /// Throws [value] if [value] is truthy. This is useful when testing the `error` argument in callbacks.
  external void ifError(Object value);

  /// Tests for any deep inequality. Opposite of [deepEqual].
  external void notDeepEqual(Object actual, Object expected, [String message]);

  /// Tests for deep strict inequality. Opposite of [deepStrictEqual].
  external void notDeepStrictEqual(Object actual, Object expected, [String message]);

  /// Tests shallow, coercive inequality with the abstract equality comparison (`!=`).
  external void notEqual(Object actual, Object expected, [String message]);

  /// Tests strict inequality as determined by the strict equality comparison (`!==`).
  external void notStrictEqual(Object actual, Object expected, [String message]);

  /// Tests if [value] is truthy.
  external void ok(Object value, [String message]);

  /// Tests strict equality as determined by the strict equality comparison (`===`).
  external void strictEqual(Object actual, Object expected, [String message]);

  /// Expects the function [block] to throw an [error].
  external void throws(void block(), [Object error, String message]);
}
