part of nodejs_interop.node;

/// The context of an assertion error.
@JS()
@anonymous
class AssertionErrorOptions {

  /// Creates a new options object.
  external factory AssertionErrorOptions({
    Object actual,
    Object expected,
    String message,
    String operator,
    Function stackStartFunction
  });

  /// The actual value.
  external Object get actual;
  external set actual(Object value);

  /// The expected value.
  external Object get expected;
  external set expected(Object value);

  /// The error message.
  external String get message;
  external set message(String value);

  /// The comparison operator used by the assertion.
  external String get operator;
  external set operator(String value);

  /// The constructor function used to customize the stack trace.
  external Function get stackStartFunction;
  external set stackStartFunction(Function value);
}

/// Indicates the failure of an assertion.
/// Such errors commonly indicate inequality of actual and expected value.
@JS(r'$assert.AssertionError')
class JSAssertionError extends JSError {

  /// Creates a new assertion error.
  external JSAssertionError(AssertionErrorOptions options);

  /// The actual value.
  external dynamic get actual;

  /// The expected value.
  external dynamic get expected;

  /// Value indicating whether the error [message] is generated.
  external bool get generatedMessage;

  /// The comparison operator used by the assertion.
  external String get operator;
}
