part of nodejs_interop.node;
// ignore_for_file: avoid_classes_with_only_static_members

/// An octet stream of binary data.
@JS('Buffer')
abstract class Buffer {

  /// The number of bytes of an element.
  external static int get BYTES_PER_ELEMENT; // ignore: non_constant_identifier_names

  /// The string value of the constructor name.
  external static String get name;

  /// This is the number of bytes used to determine the size of pre-allocated, internal [Buffer] instances used for pooling.
  /// Defaults to `8192`.
  external static int get poolSize;
  external static set poolSize(int value);

  /// Allocates a new buffer of [size] bytes, pre-filled by [fill] value (an integer, a string or a [Buffer]).
  /// If [fill] is a string, [encoding] specifies its encoding.
  external static Uint8List alloc(int size, [fill = 0, String encoding = BufferEncoding.utf8]);

  /// Allocates a new buffer of [size] bytes.
  external static Uint8List allocUnsafe(int size);

  /// Allocates a new buffer of [size] bytes.
  external static Uint8List allocUnsafeSlow(int size);

  /// Returns the actual byte length of a string [value].
  external static int byteLength(value, [String encoding = BufferEncoding.utf8]);

  /// Compares [buffer1] to [buffer2] and returns a number indicating whether [buffer1] comes before, after, or is the same as [buffer2] in sort order.
  /// Comparison is based on the actual sequence of bytes in each buffer.
  external static int compare(Uint8List buffer1, Uint8List buffer2);

  /// Returns a new buffer which is the result of concatenating all the [Buffer] or `Uint8List` instances in the [list] together.
  external static Uint8List concat(List list, [int totalLength]);

  /// Allocates a new buffer from the specified [value].
  external static Uint8List from(value, [arg1, arg2]);

  /// Gets a value indicating whether [object] is a [Buffer].
  external static bool isBuffer(object);

  /// Gets a value indicating whether [encoding] contains a supported character encoding.
  external static bool isEncoding(String encoding);
}
