part of nodejs_interop.node;

/// The character encodings currently supported by [Node.js](https://nodejs.org).
abstract class BufferEncoding {

  /// For 7-bit ASCII data only. This encoding is fast and will strip the high bit if set.
  static const String ascii = 'ascii';

  /// A special value to tell a function to return a [Buffer] instance.
  static const String buffer = 'buffer';

  /// Base64 encoding.
  /// When creating a buffer from a string, this encoding will also correctly accept [URL and Filename Safe Alphabet](https://tools.ietf.org/html/rfc4648#section-5).
  static const String base64 = 'base64';

  /// An alias for the [latin1] enumerated value.
  static const String binary = 'binary';

  /// Encode each byte as two hexadecimal characters.
  static const String hex = 'hex';

  /// A way of encoding the buffer into a [one-byte encoded string](https://tools.ietf.org/html/rfc1345).
  static const String latin1 = 'latin1';

  /// An alias for the [utf16le] enumerated value.
  static const String ucs2 = 'ucs2';

  /// 2 or 4 bytes, little-endian encoded [Unicode](https://unicode.org) characters. Surrogate pairs (U+10000 to U+10FFFF) are supported.
  static const String utf16le = 'utf16le';

  /// Multibyte encoded [Unicode](https://unicode.org) characters. Many web pages and other document formats use UTF-8.
  static const String utf8 = 'utf8';

  /// Converts the specified [encoding] to a [BufferEncoding] value.
  /// Returns [BufferEncoding.buffer] if the [encoding] is not supported.
  static String fromEncoding(Encoding encoding) {
    if (encoding is AsciiCodec) return BufferEncoding.ascii;
    if (encoding is Base64Codec) return BufferEncoding.base64;
    if (encoding is Latin1Codec) return BufferEncoding.latin1;
    if (encoding is Utf8Codec) return BufferEncoding.utf8;
    return BufferEncoding.buffer;
  }
}
