part of nodejs_interop.node;

/// Contains all public information and methods about a worker.
///
/// In the master it can be obtained using [ClusterModule.workers].
/// In a worker it can be obtained using [ClusterModule.worker].
@JS()
abstract class Worker extends EventEmitter {

  /// Value allowing to distinguish between voluntary and accidental exit.
  /// The master may choose not to respawn a worker based on this value.
  external bool get exitedAfterDisconnect;

  /// The worker identifier.
  external int get id;

  /// The underlying child process.
  external ChildProcess get process;

  /// In a worker, this method will close all servers, wait for the `close` event on those servers, and then disconnect the IPC channel.
  /// In the master, an internal message is sent to the worker causing it to call [disconnect] on itself.
  external Worker disconnect();

  /// Returns a value indicating whether the worker is connected to its master via its IPC channel.
  /// A worker is connected to its master after it has been created. It is disconnected after the `disconnect` event is emitted.
  external bool isConnected();

  /// Returns a value indicating whether the worker's process has terminated (either because of exiting or being signaled).
  external bool isDead();

  /// Kills the worker.
  ///
  /// In the master, it does this by disconnecting the [process], and once disconnected, killing with [signal].
  /// In the worker, it does it by disconnecting the channel, and then exiting with code `0`.
  external void kill([String signal = 'SIGTERM']);

  /// Sends a message to a worker or master, optionally with a handle.
  ///
  /// In the master this sends a message to a specific worker. It is identical to [ChildProcess.send].
  /// In a worker this sends a message to the master. It is identical to `process.send()`.
  external bool send(message, [sendHandleOrCallback, CompletionCallback0 callback]);
}
