part of nodejs_interop.node;

/// A simple logger with configurable output streams.
@JS('console.Console')
class Console {

  /// Creates a new console by passing one or two writable stream instances.
  ///
  /// [stdout] is a writable stream to print log or info output. [stderr] is used for warning or error output.
  /// If [stderr] is not passed, warning and error output will be sent to [stdout].
  external Console(Writable stdout, [Writable stderr]);

  /// Attempts to clear the console. When `stdout` is not a TTY, this method does nothing.
  external void clear();

  /// Maintains an internal counter specific to [label] and outputs to `stdout` the number of times [count] has been called with the given [label].
  external void count([String label = 'default']);

  /// Resets the internal counter specific to [label].
  external void countReset([String label = 'default']);

  /// An alias for the [log] method.
  external void debug(data, [Object arg1, Object arg2, Object arg3, Object arg4, Object arg5, Object arg6, Object arg7]);

  /// Uses [UtilModule.inspect] on [object] and prints the resulting string to `stdout`.
  ///
  /// This function bypasses any custom inspection function defined on [object].
  /// An optional [options] object may be passed to alter certain aspects of the formatted string.
  external void dir(Object object, [InspectOptions options]);

  /// Prints to `stderr` with newline.
  /// Multiple arguments can be passed, with the first used as the primary message and all additional used as substitution values.
  external void error([data, Object arg1, Object arg2, Object arg3, Object arg4, Object arg5, Object arg6, Object arg7]);

  /// Increases indentation of subsequent lines by two spaces.
  /// If one or more labels are provided, those are printed first without the additional indentation.
  external void group([Object label1, Object label2, Object label3, Object label4, Object label5, Object label6, Object label7, Object label8]);

  /// An alias for the [group] method.
  external void groupCollapsed();

  /// Decreases indentation of subsequent lines by two spaces.
  external void groupEnd();

  /// An alias for the [log] method.
  external void info([data, Object arg1, Object arg2, Object arg3, Object arg4, Object arg5, Object arg6, Object arg7]);

  /// Prints to `stdout` with newline.
  /// Multiple arguments can be passed, with the first used as the primary message and all additional used as substitution values.
  external void log([data, Object arg1, Object arg2, Object arg3, Object arg4, Object arg5, Object arg6, Object arg7]);

  /// Starts a JavaScript CPU profile with an optional [label] until [profileEnd] is called.
  /// The profile is then added to the **Profiles** panel of the inspector.
  /// This method does not display anything unless used in the inspector.
  external void profile([String label]);

  /// Stops the current JavaScript CPU profiling session if one has been started and prints the report to the **Profiles** panel of the inspector.
  /// This method does not display anything unless used in the inspector.
  external void profileEnd();

  /// Prints to `stdout` the specified [array] or object formatted as a table.
  /// The optional [columns] parameter selects a subset of columns to display.
  /// This method does not display anything unless used in the inspector.
  external void table(array, [List<String> columns]);

  /// Starts a timer that can be used to compute the duration of an operation.
  /// Timers are identified by a unique [label].
  external void time([String label = 'default']);

  /// Stops a timer that was previously started by calling [time] and prints the result to `stdout`.
  external void timeEnd([String label = 'default']);

  /// Adds an event with the given [label] to the **Timeline** panel of the inspector.
  /// This method does not display anything unless used in the inspector.
  external void timeStamp([String label]);

  /// Prints to `stderr` the string `"Trace:"`, followed by the formatted [message] and stack trace to the current position in the code.
  external void trace([message, Object arg1, Object arg2, Object arg3, Object arg4, Object arg5, Object arg6, Object arg7]);

  /// An alias for the [error] method.
  external void warn([data, Object arg1, Object arg2, Object arg3, Object arg4, Object arg5, Object arg6, Object arg7]);
}
