part of nodejs_interop.node;
// ignore_for_file: non_constant_identifier_names

/// The [`dns`](https://nodejs.org/api/dns.html) module.
@JS()
abstract class DnsModule extends _Resolver {

  /// Flag indicating that returned address types are determined by the types of addresses supported by the current system.
  /// For example, IPv4 addresses are only returned if the current system has at least one IPv4 address configured.
  /// Loopback addresses are not considered.
  external int get ADDRCONFIG;

  /// Error code indicating that the `GetNetworkParams` could not be found.
  external String get ADDRGETNETWORKPARAMS;

  /// Error code indicating that the address family is unsupported.
  external String get BADFAMILY;

  /// Error code indicating that illegal flags were specified.
  external String get BADFLAGS;

  /// Error code indicating that illegal hints flags were specified.
  external String get BADHINTS;

  /// Error code indicating that the hostname is misformatted.
  external String get BADNAME;

  /// Error code indicating that the DNS query is misformatted.
  external String get BADQUERY;

  /// Error code indicating that the DNS reply is misformatted.
  external String get BADRESP;

  /// Error code indicating that a string is misformatted.
  external String get BADSTR;

  /// Error code indicating that the DNS query was cancelled.
  external String get CANCELLED;

  /// Error code indicating that the DNS servers could not be contacted.
  external String get CONNREFUSED;

  /// Error code indicating that the channel is being destroyed.
  external String get DESTRUCTION;

  /// Error code indicating that an end of file was encountered.
  external String get EOF;

  /// Error code indicating that an error occurred while reading a file.
  external String get FILE;

  /// Error code indicating that the DNS server claims query was misformatted.
  external String get FORMERR;

  /// Error code indicating that an error occurred while loading the `iphlpapi.dll` library.
  external String get LOADIPHLPAPI;

  /// Error code indicating that the DNS server returned answer with no data.
  external String get NODATA;

  /// Error code indicating that the process ran out of memory.
  external String get NOMEM;

  /// Error code indicating that the given hostname is not numeric.
  external String get NONAME;

  /// Error code indicating that the domain name was not found.
  external String get NOTFOUND;

  /// Error code indicating that the DNS server does not implement requested operation.
  external String get NOTIMP;

  /// Error code indicating that the `c-ares` library initialization is not yet performed.
  external String get NOTINITIALIZED;

  /// Error code indicating that the DNS server refused query.
  external String get REFUSED;

  /// Error code indicating that the DNS server returned general failure.
  external String get SERVFAIL;

  /// Error code indicating that a timeout occurred while contacting DNS servers.
  external String get TIMEOUT;

  /// Flag indicating whether to return IPv4 mapped IPv6 addresses when a IPv6 family was specified but no IPv6 addresses were found.
  external int get V4MAPPED;

  /// Resolves a [hostname] into a `address` and `family` of the first found `A` (IPv4) or `AAAA` (IPv6) record.
  ///
  /// All `options` properties are optional. If `options` is an integer, then it must be `4` or `6`.
  /// If `options` is not provided, then IPv4 and IPv6 addresses are both returned if found.
  ///
  /// With the `all` option set to `true`, the arguments for [callback] change to `(error, addresses)`,
  /// with `addresses` being a list of [IPAddress] objects.
  external void lookup(String hostname, optionsOrCallback, [CompletionCallback1 callback]);

  /// Resolves the given [address] and [port] into a `hostname` and `service` using
  /// the operating system's underlying [`getnameinfo`](http://manpages.ubuntu.com/manpages/xenial/en/man3/getnameinfo.3.html) implementation.
  external void lookupService(String address, int port, CompletionCallback2<String, String> callback);
}

/// Options used to customize a DNS lookup.
@JS()
@anonymous
class LookupOptions {

  /// Creates a new path object.
  external factory LookupOptions({
    bool all = false,
    int family,
    int hints,
    bool verbatim = false
  });

  /// Value indicating whether to the callback receives all resolved addresses in a list, instead of a single address.
  external bool get all;
  external set all(bool value);

  /// The record family. Must be `4` or `6`. IPv4 and IPv6 addresses are both returned by default.
  external int get family;
  external set family(int value);

  /// One or more supported [`getaddrinfo`](http://manpages.ubuntu.com/manpages/xenial/en/man3/getaddrinfo.3.html) flags.
  external int get hints;
  external set hints(int value);

  /// Value indicating whether the callback receives IPv4 and IPv6 addresses
  /// in the order the DNS resolver returned them, instead of reordering them.
  external bool get verbatim;
  external set verbatim(bool value);
}
