part of nodejs_interop.node;

/// The common super class for DNS records.
@JS()
abstract class DnsRecord {

  /// A string identifying the record type.
  external String get type;
}

/// Represents an IPv4 (`A` record) or IPv6 (`AAAA` record) address.
@JS()
abstract class AddressRecord extends DnsRecord {

  /// The IP address.
  external String get address;

  /// The "Time-To-Live" value.
  external int get ttl;
}

/// Represents a mail exchanger record (`MX` record).
@JS()
abstract class MXRecord extends DnsRecord {

  /// The hostname of the machine providing the service.
  external String get exchange;

  /// The record priority.
  external int get priority;
}

/// Represents a hostname record (`CNAME`, `NS` or `PTR` record).
@JS()
abstract class NameRecord extends DnsRecord {

  /// The hostname.
  external String get value;
}

/// Represents a naming authority pointer (`NAPTR` record).
@JS()
abstract class NaptrRecord extends DnsRecord {

  /// Flags to control aspects of the rewriting and interpretation of the fields in the record.
  external String get flags;

  /// The order in which the record must be processed.
  external int get order;

  /// A relative weight for records with the same [order].
  external int get preference;

  /// A substitution expression that is applied to the original string held by the client
  /// in order to construct the next domain name to lookup.
  external String get regexp;

  /// A domain name which is the next domain to query for depending on the potential values found in the [flags] field.
  external String get replacement;

  /// The service parameters applicable to this delegation path.
  external String get service;
}

/// Represents a start of authority record (`SOA` record).
@JS()
abstract class SoaRecord extends DnsRecord {

  /// The expiry period.
  external int get expire;

  /// The email address of the administrator.
  external String get hostmaster;

  /// The minimum "Time-To-Live" value.
  external int get minttl;

  /// The primary master name server.
  external String get nsname;

  /// The refresh interval.
  external int get refresh;

  /// The retry interval.
  external int get retry;

  /// The serial number.
  external int get serial;
}

/// Represents a service locator (`SRV` record).
@JS()
abstract class SrvRecord extends DnsRecord {

  /// The hostname of the machine providing the service.
  external int get name;

  /// The TCP or UDP port on which the service is to be found.
  external int get port;

  /// The priority of the target host.
  external int get priority;

  /// A relative weight for records with the same [priority].
  external int get weight;
}

/// Represents a text query (`TXT` record).
@JS()
abstract class TxtRecord extends DnsRecord {

  /// The query entries.
  external List<String> get entries;
}
