part of nodejs_interop.node;

/// An object (called "emitter") periodically emitting named events that cause registered functions ("listeners") to be called.
@JS(r'$events')
class EventEmitter {

  /// Creates a new event emitter.
  external EventEmitter();

  /// The default maximum number of listeners. Defaults to `10`.
  external static int get defaultMaxListeners;
  external static set defaultMaxListeners(int value);

  /// An alias for the [on] method.
  external EventEmitter addListener(eventName, Function listener);

  /// Synchronously calls each of the listeners registered for the event named [eventName],
  /// in the order they were registered, passing the supplied arguments to each.
  /// Returns `true` if the event had listeners, `false` otherwise.
  external bool emit(eventName, [Object arg1, Object arg2, Object arg3, Object arg4, Object arg5, Object arg6, Object arg7]);

  /// Gets a list listing the events for which the emitter has registered listeners. The values in the list will be strings or symbols.
  external List eventNames();

  /// Gets the maximum number of listeners. Defaults to [defaultMaxListeners].
  external int getMaxListeners();

  /// Gets the number of listeners listening to the event named [eventName].
  external int listenerCount(eventName);

  /// Gets a copy of the listener list for the event named [eventName].
  external List<Function> listeners(eventName);

  /// Adds the [listener] function to the end of the listener list for the event named [eventName].
  external EventEmitter on(eventName, Function listener);

  /// Adds a one time [listener] function for the event named [eventName].
  /// The next time [eventName] is triggered, this [listener] is removed and then invoked.
  external EventEmitter once(eventName, Function listener);

  /// Adds the [listener] function to the beginning of the listener list for the event named [eventName].
  external EventEmitter prependListener(eventName, Function listener);

  /// Adds a one time [listener] function for the event named [eventName] to the beginning of the listener list.
  /// The next time [eventName] is triggered, this listener is removed, and then invoked.
  external EventEmitter prependOnceListener(eventName, Function listener);

  /// Removes all listeners, or those of the specified [eventName].
  external EventEmitter removeAllListeners([eventName]);

  /// Removes the specified [listener] from the listener list for the event named [eventName].
  ///
  /// Will remove, at most, one instance of a listener from the listener list.
  /// If any single listener has been added multiple times to the listener list for the specified [eventName],
  /// then [removeListener] must be called multiple times to remove each instance.
  external EventEmitter removeListener(eventName, Function listener);

  /// Sets the maximum [number] of listeners.
  external EventEmitter setMaxListeners(int number);
}
