part of nodejs_interop.node;
// ignore_for_file: non_constant_identifier_names

/// Commonly used constants for file system operations.
@JS()
abstract class FSConstants {

  /// Flag indicating that the file is visible to the calling process.
  external int get F_OK;

  /// Flag indicating that the file can be read by the calling process.
  external int get R_OK;

  /// Flag indicating that the file can be written by the calling process.
  external int get W_OK;

  /// Flag indicating that the file can be executed by the calling process.
  external int get X_OK;

  /// Flag indicating that data will be appended to the end of the file.
  external int get O_APPEND;

  /// Flag indicating to create the file if it does not already exist.
  external int get O_CREAT;

  /// When set, an attempt will be made to minimize caching effects of file I/O.
  external int get O_DIRECT;

  /// Flag indicating that the open should fail if the path is not a directory.
  external int get O_DIRECTORY;

  /// Flag indicating that the file is opened for synchronized I/O with write operations waiting for data integrity.
  external int get O_DSYNC;

  /// Flag indicating that opening a file should fail if the O_CREAT flag is set and the file already exists.
  external int get O_EXCL;

  /// Flag indicating reading accesses to the file system will no longer result in an update to the atime information associated with the file.
  /// This flag is available on Linux operating systems only.
  external int get O_NOATIME;

  /// Flag indicating that if path identifies a terminal device, opening the path shall not cause that terminal
  /// to become the controlling terminal for the process (if the process does not already have one).
  external int get O_NOCTTY;

  /// Flag indicating that the open should fail if the path is a symbolic link.
  external int get O_NOFOLLOW;

  /// Flag indicating to open the file in nonblocking mode when possible.
  external int get O_NONBLOCK;

  /// Flag indicating to open a file for read-only access.
  external int get O_RDONLY;

  /// Flag indicating to open a file for read-write access.
  external int get O_RDWR;

  /// Flag indicating to open the symbolic link itself rather than the resource it is pointing to.
  external int get O_SYMLINK;

  /// Flag indicating that the file is opened for synchronized I/O with write operations waiting for file integrity.
  external int get O_SYNC;

  /// Flag indicating that if the file exists and is a regular file, and the file is opened successfully for write access,
  /// its length shall be truncated to zero.
  external int get O_TRUNC;

  /// Flag indicating to open a file for write-only access.
  external int get O_WRONLY;

  /// File type constant for a block-oriented device file.
  external int get S_IFBLK;

  /// File type constant for a character-oriented device file.
  external int get S_IFCHR;

  /// File type constant for a directory.
  external int get S_IFDIR;

  /// File type constant for a FIFO/pipe.
  external int get S_IFIFO;

  /// File type constant for a symbolic link.
  external int get S_IFLNK;

  /// Bit mask used to extract the file type code.
  external int get S_IFMT;

  /// File type constant for a regular file.
  external int get S_IFREG;

  /// File type constant for a socket.
  external int get S_IFSOCK;

  /// File mode indicating readable by group.
  external int get S_IRGRP;

  /// File mode indicating readable by others.
  external int get S_IROTH;

  /// File mode indicating readable by owner.
  external int get S_IRUSR;

  /// File mode indicating writable by group.
  external int get S_IWGRP;

  /// File mode indicating writable by others.
  external int get S_IWOTH;

  /// File mode indicating writable by owner.
  external int get S_IWUSR;

  /// File mode indicating readable, writable and executable by group.
  external int get S_IRWXG;

  /// File mode indicating readable, writable and executable by others.
  external int get S_IRWXO;

  /// File mode indicating readable, writable and executable by owner.
  external int get S_IRWXU;

  /// File mode indicating executable by group.
  external int get S_IXGRP;

  /// File mode indicating executable by others.
  external int get S_IXOTH;

  /// File mode indicating executable by owner.
  external int get S_IXUSR;
}
