part of nodejs_interop.node;

/// Options used to customize the creation of a file stream.
@JS()
@anonymous
class CreateStreamOptions {

  /// Creates a new options object.
  external factory CreateStreamOptions({
    bool autoClose = true,
    String encoding,
    int end,
    int fd,
    String flags,
    int mode = 0x1B6,
    int start
  });

  /// Value indicating whether the file descriptor will be closed automatically.
  external bool get autoClose;
  external set autoClose(bool value);

  /// The encoding of the stream contents.
  external String get encoding;
  external set encoding(String value);

  /// The file position where to end.
  /// Not supported by the [FSModule.createWriteStream] method.
  external int get end;
  external set end(int value);

  /// A file descriptor. If specified, the file path will be ignored.
  external int get fd;
  external set fd(int value);

  /// Value indicating the behavior of the file to be opened.
  external String get flags;
  external set flags(String value);

  /// The file permissions.
  external int get mode;
  external set mode(int value);

  /// The file position where to start.
  external int get start;
  external set start(int value);
}

/// Options used to interpret the character encoding of strings.
@JS()
@anonymous
class EncodingOptions {

  /// Creates a new options object.
  external factory EncodingOptions({String encoding = BufferEncoding.utf8});

  /// The character encoding.
  external String get encoding;
  external set encoding(String value);
}

/// Options used to customize the file reading.
@JS()
@anonymous
class ReadFileOptions {

  /// Creates a new options object.
  external factory ReadFileOptions({String encoding, String flag = 'r'});

  /// The encoding of the returned contents. If no encoding is specified, then the raw buffer is returned.
  external String get encoding;
  external set encoding(String value);

  /// Value indicating the behavior of the file to be opened.
  external String get flag;
  external set flag(String value);
}

/// Options used to customize a file watcher.
@JS()
@anonymous
class WatchFileOptions {

  /// Creates a new options object.
  external factory WatchFileOptions({
    int interval = 5007,
    bool persistent = true
  });

  /// Value indicating how often the target should be polled in milliseconds.
  external int get interval;
  external set interval(int value);

  /// Whether the process should continue to run as long as the file is being watched.
  external bool get persistent;
  external set persistent(bool value);
}

/// Options used to customize a file system entity watcher.
@JS()
@anonymous
class WatchOptions {

  /// Creates a new options object.
  external factory WatchOptions({
    String encoding = BufferEncoding.utf8,
    bool persistent = true,
    bool recursive = false
  });

  /// The character encoding to be used for the `filename` passed to the listener.
  external String get encoding;
  external set encoding(String value);

  /// Whether the process should continue to run as long as files are being watched.
  external bool get persistent;
  external set persistent(bool value);

  /// Whether all subdirectories should be watched, or only the current directory.
  /// This applies when a directory is specified, and only on supported platforms.
  external bool get recursive;
  external set recursive(bool value);
}

/// Options used to customize the file writing.
@JS()
@anonymous
class WriteFileOptions {

  /// Creates a new options object.
  external factory WriteFileOptions({String encoding = BufferEncoding.utf8, String flag, int mode = 0x1B6});

  /// The encoding of the written contents.
  external String get encoding;
  external set encoding(String value);

  /// Value indicating the behavior of the file to be opened.
  external String get flag;
  external set flag(String value);

  /// The file permissions.
  external int get mode;
  external set mode(int value);
}
