part of nodejs_interop.node;

/// Represents the result of calling the POSIX [`stat`](http://manpages.ubuntu.com/manpages/xenial/man2/stat.2.html) function on a file system entity.
/// It is an immutable object, representing the snapshotted values returned by the `stat()` call.
@JS()
abstract class Stats {

  /// The time of the last access to the data of the file system entity.
  /// On Windows platforms, this may have 1 day granularity, and be out of date by an hour.
  external DateTime get atime;

  /// The timestamp, in milliseconds, of the last access to the data of the file system entity.
  external double get atimeMs;

  /// The time of the creation of file system object.
  external DateTime get birthtime;

  /// The timestamp, in milliseconds, of the creation of file system object.
  external double get birthtimeMs;

  /// The number of number of 512B blocks allocated.
  external int get blocks;

  /// The block size for file system I/O.
  external int get blksize;

  /// The time of the last change to the data or metadata of the file system object.
  /// On Windows platforms, this is instead the file creation time.
  external DateTime get ctime;

  /// The timestamp, in milliseconds, of the last change to the data or metadata of the file system object.
  external double get ctimeMs;

  /// Identifier of the device containing the file system entity.
  external int get dev;

  /// The group identifier of the file owner.
  external int get gid;

  /// The inode number.
  external int get ino;

  /// The mode of the file system entity. Permissions are encoded in the lower 16 bits of this number.
  external int get mode;

  /// The time of the last change to the data of the file system object.
  external DateTime get mtime;

  /// The timestamp, in milliseconds, of the last change to the data of the file system object.
  external double get mtimeMs;

  /// The number of hard links.
  external int get nlink;

  /// The device identifier for a special file system entity.
  external int get rdev;

  /// The user identifier of the file owner.
  external int get uid;

  /// The total size, in bytes, of the file system entity.
  external int get size;

  /// Gets a value indicating whether the file system entity is a block device.
  external bool isBlockDevice();

  /// Gets a value indicating whether the file system entity is a block device.
  external bool isCharacterDevice();

  /// Gets a value indicating whether the file system entity is a directory.
  external bool isDirectory();

  /// Gets a value indicating whether the file system entity is a file.
  external bool isFile();

  /// Gets a value indicating whether the file system entity is a symbolic link.
  external bool isSymbolicLink();

  /// Gets a value indicating whether the file system entity is a named pipe (i.e. FIFO).
  external bool isFIFO();

  /// Gets a value indicating whether the file system entity is a socket.
  external bool isSocket();
}
