part of nodejs_interop.node;

/// A function invoked when an operation is finally completed.
typedef void CompletionCallback0([JSError error]);

/// A function invoked with a value when an operation is finally completed.
typedef void CompletionCallback1<T>(JSError error, [T value]);

/// A function invoked with two values when an operation is finally completed.
typedef void CompletionCallback2<T1, T2>(JSError error, [T1 value1, T2 value2]);

/// Converts the specified [readable] stream to a [Stream].
Stream<T> asStream<T>(Readable<T> readable) {
  var controller = new StreamController<T>();
  readable
    ..on('data', allowInterop((data) => controller.add(data)))
    ..on('end', allowInterop(() => controller.close()))
    ..on('error', allowInterop((error) => controller..addError(error)..close()));

  return controller.stream;
}

/// Converts the given function following the Node.js callback style to a [Future].
Future<Null> fn1cb0ToFuture(void fn(arg1, CompletionCallback0 callback), arg1) {
  var completer = new Completer<Null>();
  fn(arg1, allowInterop(([error]) {
    if (error == null) completer.complete();
    else completer.completeError(error);
  }));

  return completer.future;
}

/// Converts the given function following the Node.js callback style to a [Future].
Future<T> fn1cb1ToFuture<T>(void fn(arg1, CompletionCallback1<T> callback), arg1) {
  var completer = new Completer<T>();
  fn(arg1, allowInterop((error, [value]) {
    if (error == null) completer.complete(value);
    else completer.completeError(error);
  }));

  return completer.future;
}

/// Converts the given function following the Node.js callback style to a [Future].
Future<Tuple2<T1, T2>> fn1cb2ToFuture<T1, T2>(void fn(arg1, CompletionCallback2<T1, T2> callback), arg1) {
  var completer = new Completer<Tuple2<T1, T2>>();
  fn(arg1, allowInterop((error, [value1, value2]) {
    if (error == null) completer.complete(new Tuple2<T1, T2>(value1, value2));
    else completer.completeError(error);
  }));

  return completer.future;
}

/// Converts the given function following the Node.js callback style to a [Future].
Future<Null> fn2cb0ToFuture(void fn(arg1, arg2, CompletionCallback0 callback), arg1, arg2) {
  var completer = new Completer<Null>();
  fn(arg1, arg2, allowInterop(([error]) {
    if (error == null) completer.complete();
    else completer.completeError(error);
  }));

  return completer.future;
}

/// Converts the given function following the Node.js callback style to a [Future].
Future<T> fn2cb1ToFuture<T>(void fn(arg1, arg2, CompletionCallback1<T> callback), arg1, arg2) {
  var completer = new Completer<T>();
  fn(arg1, arg2, allowInterop((error, [value]) {
    if (error == null) completer.complete(value);
    else completer.completeError(error);
  }));

  return completer.future;
}

/// Converts the given function following the Node.js callback style to a [Future].
Future<Tuple2<T1, T2>> fn2cb2ToFuture<T1, T2>(void fn(arg1, arg2, CompletionCallback2<T1, T2> callback), arg1, arg2) {
  var completer = new Completer<Tuple2<T1, T2>>();
  fn(arg1, arg2, allowInterop((error, [value1, value2]) {
    if (error == null) completer.complete(new Tuple2<T1, T2>(value1, value2));
    else completer.completeError(error);
  }));

  return completer.future;
}

/// Converts the given function following the Node.js callback style to a [Future].
Future<Null> fn3cb0ToFuture(void fn(arg1, arg2, arg3, CompletionCallback0 callback), arg1, arg2, arg3) {
  var completer = new Completer<Null>();
  fn(arg1, arg2, arg3, allowInterop(([error]) {
    if (error == null) completer.complete();
    else completer.completeError(error);
  }));

  return completer.future;
}

/// Converts the given function following the Node.js callback style to a [Future].
Future<T> fn3cb1ToFuture<T>(void fn(arg1, arg2, arg3, CompletionCallback1<T> callback), arg1, arg2, arg3) {
  var completer = new Completer<T>();
  fn(arg1, arg2, arg3, allowInterop((error, [value]) {
    if (error == null) completer.complete(value);
    else completer.completeError(error);
  }));

  return completer.future;
}

/// Converts the given function following the Node.js callback style to a [Future].
Future<Tuple2<T1, T2>> fn3cb2ToFuture<T1, T2>(void fn(arg1, arg2, arg3, CompletionCallback2<T1, T2> callback), arg1, arg2, arg3) {
  var completer = new Completer<Tuple2<T1, T2>>();
  fn(arg1, arg2, arg3, allowInterop((error, [value1, value2]) {
    if (error == null) completer.complete(new Tuple2<T1, T2>(value1, value2));
    else completer.completeError(error);
  }));

  return completer.future;
}

/// Converts the given function following the Node.js callback style to a [Future].
Future<Null> fn4cb0ToFuture(void fn(arg1, arg2, arg3, arg4, CompletionCallback0 callback), arg1, arg2, arg3, arg4) {
  var completer = new Completer<Null>();
  fn(arg1, arg2, arg3, arg4, allowInterop(([error]) {
    if (error == null) completer.complete();
    else completer.completeError(error);
  }));

  return completer.future;
}

/// Converts the given function following the Node.js callback style to a [Future].
Future<T> fn4cb1ToFuture<T>(void fn(arg1, arg2, arg3, arg4, CompletionCallback1<T> callback), arg1, arg2, arg3, arg4) {
  var completer = new Completer<T>();
  fn(arg1, arg2, arg3, arg4, allowInterop((error, [value]) {
    if (error == null) completer.complete(value);
    else completer.completeError(error);
  }));

  return completer.future;
}

/// Converts the given function following the Node.js callback style to a [Future].
Future<Tuple2<T1, T2>> fn4cb2ToFuture<T1, T2>(void fn(arg1, arg2, arg3, arg4, CompletionCallback2<T1, T2> callback), arg1, arg2, arg3, arg4) {
  var completer = new Completer<Tuple2<T1, T2>>();
  fn(arg1, arg2, arg3, arg4, allowInterop((error, [value1, value2]) {
    if (error == null) completer.complete(new Tuple2<T1, T2>(value1, value2));
    else completer.completeError(error);
  }));

  return completer.future;
}

/// Converts the given function following the Node.js callback style to a [Future].
Future<Null> fn5cb0ToFuture(void fn(arg1, arg2, arg3, arg4, arg5, CompletionCallback0 callback), arg1, arg2, arg3, arg4, arg5) {
  var completer = new Completer<Null>();
  fn(arg1, arg2, arg3, arg4, arg5, allowInterop(([error]) {
    if (error == null) completer.complete();
    else completer.completeError(error);
  }));

  return completer.future;
}

/// Converts the given function following the Node.js callback style to a [Future].
Future<T> fn5cb1ToFuture<T>(void fn(arg1, arg2, arg3, arg4, arg5, CompletionCallback1<T> callback), arg1, arg2, arg3, arg4, arg5) {
  var completer = new Completer<T>();
  fn(arg1, arg2, arg3, arg4, arg5, allowInterop((error, [value]) {
    if (error == null) completer.complete(value);
    else completer.completeError(error);
  }));

  return completer.future;
}

/// Converts the given function following the Node.js callback style to a [Future].
Future<Tuple2<T1, T2>> fn5cb2ToFuture<T1, T2>(void fn(arg1, arg2, arg3, arg4, arg5, CompletionCallback2<T1, T2> callback), arg1, arg2, arg3, arg4, arg5) {
  var completer = new Completer<Tuple2<T1, T2>>();
  fn(arg1, arg2, arg3, arg4, arg5, allowInterop((error, [value1, value2]) {
    if (error == null) completer.complete(new Tuple2<T1, T2>(value1, value2));
    else completer.completeError(error);
  }));

  return completer.future;
}

/// Requires the JavaScript module with the specified [identifier].
///
/// Throws a [DeferredLoadException] if the module could not be required.
/// If a [onError] handler is provided, it is called with the [identifier] as argument, and its return value is used instead.
///
/// This function is prefered to the [require] function because it properly exposes to the Dart engine the classes exported by the required module.
T loadLibrary<T>(String identifier, {T onError(String identifier)}) {
  try {
    T module = require(identifier);
    var moduleId = '\$${identifier.replaceAll(new RegExp('[-/@.]'), '_')}';
    if (!hasProperty(global, moduleId)) setProperty(global, moduleId, module);
    return module;
  }

  on JSError {
    if (onError != null) return onError(identifier);
    throw new DeferredLoadException(identifier);
  }
}
