part of nodejs_interop.node;

/// The [`Module`](https://nodejs.org/api/modules.html) class representing a JavaScript module.
@JS()
abstract class Module {

  /// The module objects required by this one.
  external List<Module> get children;

  /// The [exports] object is created by the module system.
  /// Sometimes this is not acceptable; many want their module to be an instance of some class.
  /// To do this, assign the desired export object to [exports].
  external dynamic get exports;
  external set exports(value);

  /// The fully resolved filename to the module.
  external String get filename;

  /// The identifier for the module. Typically this is the fully resolved filename.
  external String get id;

  /// Whether or not the module is done loading, or is in the process of loading.
  external bool get loaded;

  /// The module that first required this one.
  external Module get parent;

  /// The search paths for the module.
  external List<String> get paths;

  /// Provides a way to load a module as if [require] was called from the original module.
  ///
  /// In order to do this, it is necessary to get a reference to the [module] object.
  /// Since [require] returns the [exports], and the [module] is typically only available within a specific module's code,
  /// it must be explicitly exported in order to be used.
  external dynamic require(String identifier);
}
