part of nodejs_interop.node;

/// A TCP socket or a streaming IPC endpoint.
@JS(r'$net.Socket')
class Socket extends Duplex /* TODO <generic ???> */ {

  /// Creates a new TCP socket or streaming IPC endpoint.
  external Socket([SocketOptions options]);

  /// The number of characters currently buffered to be written.
  external int get bufferSize;

  /// The amount of received bytes.
  external int get bytesRead;

  /// The amount of bytes sent.
  external int get bytesWritten;

  /// Returns the bound address, the address family name and port of the socket as reported by the operating system.
  external SocketAddress address();
}

/// An endpoint to which a socket is connected.
@JS()
abstract class SocketAddress {

  /// The bound address.
  external String get address;

  /// The address family name.
  external String get family;

  /// The port of the socket.
  external int get port;
}

/// Options used to customize the creation of a [Socket].
@JS()
@anonymous
class SocketOptions {

  /// Creates a new options object.
  external factory SocketOptions({
    bool allowHalfOpen = false,
    int fd,
    bool readable = false,
    bool writable = false
  });

  /// Value indicating whether half-opened TCP connections are allowed.
  external bool get allowHalfOpen;
  external set allowHalfOpen(bool value);

  /// If specified, wraps around an existing socket with the given file descriptor, otherwise a new socket will be created.
  external int get fd;
  external set fd(int value);

  /// Value indicating whether to allow reads on the socket when an [fd] is passed.
  external bool get readable;
  external set readable(bool value);

  /// Value indicating whether to allow writes on the socket when an [fd] is passed.
  external bool get writable;
  external set writable(bool value);
}
