part of nodejs_interop.node;

/// The [`querystring`](https://nodejs.org/api/querystring.html) module.
@JS()
abstract class QueryStringModule {

  /// Performs URL percent-encoding on the given [string] in a manner that is optimized for the specific requirements of URL query strings.
  external String escape(String string);

  /// Parses a URL query [string] into a collection of key and value pairs.
  ///
  /// The `separator` argument is a substring used to delimit key and value pairs in the query string (defaults to `"&"`).
  /// The `equal` argument is a substring used to delimit keys and values in the query string (defaults to `"="`).
  external dynamic parse(String string, [separatorOrEqual, equalOrOptions, ParseOptions options]);

  /// Produces a URL query string from a given [object] by iterating through its own properties.
  ///
  /// The `separator` argument is a substring used to delimit key and value pairs in the query string (defaults to `"&"`).
  /// The `equal` argument is a substring used to delimit keys and values in the query string (defaults to `"="`).
  external String stringify(Object object, [separatorOrEqual, equalOrOptions, StringifyOptions options]);

  /// Performs decoding of URL percent-encoded characters on the given [string].
  external String unescape(String string);
}

/// Provides options for parsing a URL query string.
@JS()
@anonymous
class ParseOptions {

  /// Creates a new options object.
  external factory ParseOptions({String decodeURIComponent(String value), int maxKeys = 1000});

  /// The function to use when decoding percent-encoded characters in the query string.
  /// Defaults to [QueryStringModule.unescape].
  external Function get decodeURIComponent;
  external set decodeURIComponent(String fn(String value));

  /// The maximum number of keys to parse.
  /// Specify `0` to remove key counting limitations.
  external int get maxKeys;
  external set maxKeys(int value);
}

/// Provides options for serializing an object into a URL query string.
@JS()
@anonymous
class StringifyOptions {

  /// Creates a new options object.
  external factory StringifyOptions({String encodeURIComponent(String value)});

  /// The function to use when converting URL-unsafe characters to percent-encoding in the query string.
  /// Defaults to [QueryStringModule.escape].
  external Function get encodeURIComponent;
  external set encodeURIComponent(String fn(String value));
}
