part of nodejs_interop.node;

/// A [Duplex] stream where the output is in some way related to the input.
@JS(r'$stream.Transform')
class Transform<T> extends Duplex<T> {

  /// Creates a new transform stream.
  external Transform([TransformOptions options]);
}

/// Options passed to transform streams.
@JS()
@anonymous
class TransformOptions extends DuplexOptions {

  /// Creates a new path object.
  external factory TransformOptions({
    void flush(CompletionCallback1 callback),
    void transform(chunk, String encoding, CompletionCallback1 callback),
    bool allowHalfOpen = true,
    bool decodeStrings = true,
    void destroy(JSError error, CompletionCallback0 callback),
    String encoding,
    int highWaterMark,
    bool objectMode = false,
    void read(int size),
    int readableHighWaterMark,
    bool readableObjectMode = false,
    int writableHighWaterMark,
    bool writableObjectMode = false,
    void write(chunk, String encoding, CompletionCallback0 callback),
    void writev(List chunks, CompletionCallback0 callback)
  });

  /// Implementation for the `_flush()` method.
  external Function get flush;
  external set flush(void fn(CompletionCallback1 callback));

  /// Implementation for the `_transform()` method.
  external Function get transform;
  external set transform(void fn(chunk, String encoding, CompletionCallback1 callback));
}

/// A trivial implementation of a [Transform] stream that simply passes the input bytes across to the output.
@JS(r'$stream.PassThrough')
class PassThrough<T> extends Transform<T> {

  /// Creates a new passthrough stream.
  external PassThrough([TransformOptions options]);
}
