part of nodejs_interop.node;

/// A writable stream is an abstraction for a destination to which data is written.
@JS(r'$stream.Writable')
class Writable<T> extends EventEmitter {

  /// Creates a new writable stream.
  external Writable([WritableOptions options]);

  /// The value of [WritableOptions.highWaterMark] passed when constructing this stream.
  external int get writableHighWaterMark; // TODO: Node.js 9.3+

  /// Forces all written data to be buffered in memory.
  /// The buffered data will be flushed when either the [uncork] or [end] methods are called.
  external void cork();

  /// Destroys this stream, and emits the passed [error].
  /// After this call, the stream will release any internal resources.
  external Writable<T> destroy([JSError error]);

  /// Signals that no more data will be written to the [Writable].
  ///
  /// The optional [chunk] and `encoding` arguments allow one final additional chunk of data to be written immediately before closing the stream.
  /// For streams not operating in object mode, [chunk] must be a string, `Buffer` or `Uint8List`.
  /// For object mode streams, [chunk] may be any value other than `null`.
  ///
  /// If provided, the optional [callback] function is attached as a listener for the `finish` event.
  external void end([T chunk, encodingOrCallback, void callback()]);

  /// Sets the default [encoding] for this [Writable] stream.
  external Writable<T> setDefaultEncoding(String encoding);

  /// Flushes all data buffered since [cork] was called.
  external void uncork();

  /// Writes some data to the stream, and calls the supplied [callback] once the data has been fully handled.
  ///
  /// If an error occurs, the callback may or may not be called with the error as its first argument.
  /// To reliably detect write errors, add a listener for the `error` event.
  ///
  /// The return value is `true` if the internal buffer is less than the [WritableOptions.highWaterMark] configured
  /// when the stream was created after admitting [chunk]. If `false` is returned, further attempts to write data
  /// to the stream should stop until the `drain` event is emitted.
  external bool write(T chunk, [encodingOrCallback, CompletionCallback0 callback]);
}

/// Options passed to writable streams.
@JS()
@anonymous
class WritableOptions {

  /// Creates a new path object.
  external factory WritableOptions({
    bool decodeStrings = true,
    void destroy(JSError error, CompletionCallback0 callback),
    int highWaterMark,
    bool objectMode = false,
    void write(chunk, String encoding, CompletionCallback0 callback),
    void writev(List chunks, CompletionCallback0 callback)
  });

  /// Value indicating whether to decode strings into `Buffer` before passing them to [Writable.write].
  external bool get decodeStrings;
  external set decodeStrings(bool value);

  /// Implementation for the `_destroy()` method.
  external Function get destroy;
  external set destroy(void fn(JSError error, CompletionCallback0 callback));

  /// Buffer level when [Writable.write] starts returning `false`.
  /// Defaults to `16384` (16kb), or `16` for streams in object mode.
  external int get highWaterMark;
  external set highWaterMark(int value);

  /// Value indicating whether the stream operates in object mode.
  external bool get objectMode;
  external set objectMode(bool value);

  /// Implementation for the `_write()` method.
  external Function get write;
  external set write(void fn(chunk, String encoding, CompletionCallback0 callback));

  /// Implementation for the `_writev()` method.
  external Function get writev;
  external set writev(void fn(List chunks, CompletionCallback0 callback));
}
