part of nodejs_interop.node;

/// A [Socket] that represents the readable side of a TTY.
/// In normal circumstances `process.stdin` will be the only [TtyReadStream] instance in a Node.js process
/// and there should be no reason to create additional instances.
@JS()
abstract class TtyReadStream extends Socket {

  /// Value indicating whether the TTY is currently configured to operate as a raw device.
  external bool get isRaw;

  /// Value indicating whether the stream is associated with a TTY.
  external bool get isTTY;

  /// Configures the TTY to operate as a raw device.
  external void setRawMode(bool mode); // ignore: avoid_positional_boolean_parameters
}

/// A [Socket] that represents the writable side of a TTY.
/// In normal circumstances, `process.stdout` and `process.stderr` will be the only [TtyWriteStream] instances created for a Node.js process
/// and there should be no reason to create additional instances.
@JS()
abstract class TtyWriteStream extends Socket {

  /// The number of columns the TTY currently has.
  external int get columns;

  /// Value indicating whether the stream is associated with a TTY.
  external bool get isTTY;

  /// The number of rows the TTY currently has.
  external int get rows;
}
