part of nodejs_interop.node;

/// Represents a URL.
@JS(r'$url.URL')
class Url {

  /// Creates a new URL, using the specified [base] URL to resolve against if the [input] is not absolute.
  external Url(String input, [base]);

  /// The fragment portion of the URL.
  external String get hash;
  external set hash(String value);

  /// The host portion of the URL.
  external String get host;
  external set host(String value);

  /// The hostname portion of the URL. The key difference between [host] and [hostname] is that [hostname] does not include the [port].
  external String get hostname;
  external set hostname(String value);

  /// The serialized URL.
  external String get href;
  external set href(String value);

  /// The read-only serialization of the URL's origin.
  external String get origin;

  /// The password portion of the URL.
  external String get password;
  external set password(String value);

  /// The path portion of the URL.
  external String get pathname;
  external set pathname(String value);

  /// The port portion of the URL.
  external int get port;
  external set port(int value);

  /// The protocol portion of the URL.
  external String get protocol;
  external set protocol(String value);

  /// The serialized query portion of the URL.
  external String get search;
  external set search(String value);

  /// The query parameters of the URL.
  external UrlSearchParams get searchParams;

  /// The username portion of the URL.
  external String get username;
  external set username(String value);

  /// Converts this object to a JSON representation.
  external String toJSON();
}

/// Defines utility methods to work with the query string of a URL.
@JS(r'$url.URLSearchParams')
class UrlSearchParams {

  /// Creates new search parameters.
  external UrlSearchParams([query]);

  /// Appends a new [name]-[value] pair to the query string.
  external void append(String name, String value);

  /// Removes all name-value pairs whose name is [name].
  external void delete(String name);

  /// Returns an iterator over each of the name-value pairs in the query.
  external Iterable<List> entries();

  /// Iterates over each name-value pair in the query and invokes the given function.
  external void forEach(void fn(String value, String name, UrlSearchParams searchParams), [Object thisArg]);

  /// Returns the value of the first name-value pair whose name is [name]. If there are no such pairs, `null` is returned.
  external String get(String name);

  /// Returns the values of all name-value pairs whose name is [name]. If there are no such pairs, an empty list is returned.
  external List<String> getAll(String name);

  /// Returns a vale indicating whether there is at least one name-value pair whose name is [name].
  external bool has(String name);

  /// Returns an iterator over the names of each name-value pair.
  external Iterable<String> keys();

  /// Returns an iterator over the values of each name-value pair.
  external Iterable<String> values();

  /// Associates the specified [value] with the given [name].
  /// If there are any pre-existing name-value pairs whose names are [name], sets the first such pair's value to [value] and remove all others.
  external void set(String name, String value);

  /// Sorts all existing name-value pairs in-place by their names.
  external void sort();
}
