part of nodejs_interop.node;

/// An object whose properties represent significant elements of a URL.
@JS()
@anonymous
class UrlObject {

  /// Creates a new URL object.
  external factory UrlObject({
    String auth,
    String hash,
    String host,
    String hostname,
    String href,
    String path,
    String pathname,
    int port,
    String protocol,
    query,
    String search,
    bool slashes
  });

  /// The username and password portion of the URL, also referred to as "userinfo".
  external String get auth;
  external set auth(String value);

  /// The "fragment" portion of the URL including the leading hash (`#`) character.
  external String get hash;
  external set hash(String value);

  /// The full lower-cased host portion of the URL, including the [port] if specified.
  external String get host;
  external set host(String value);

  /// The lower-cased host name portion of the [host] component without the [port] included.
  external String get hostname;
  external set hostname(String value);

  /// The full URL string that was parsed with both the [protocol] and [host] components converted to lower-case.
  external String get href;
  external set href(String value);

  /// A concatenation of the [pathname] and [search] components.
  external String get path;
  external set path(String value);

  /// The entire path section of the URL.
  external String get pathname;
  external set pathname(String value);

  /// The numeric port portion of the [host] component.
  external int get port;
  external set port(int value);

  /// The URL's lower-cased protocol scheme.
  external String get protocol;
  external set protocol(String value);

  /// Either the query string without the leading question mark (`?`), or an object returned by the [QueryStringModule.parse] method.
  external dynamic get query;
  external set query(value);

  /// The entire "query string" portion of the URL, including the leading question mark (`?`) character.
  external String get search;
  external set search(String value);

  /// Value indicating whether two forward-slash characters (`/`) are required following the colon in the [protocol].
  external bool get slashes;
  external set slashes(bool value);
}
