part of nodejs_interop.node;

/// Inspects objects by returning a string representation that is primarily useful for debugging.
@JS()
abstract class InspectFunction implements Function {

  /// An object associating color names to their value.
  external dynamic get colors;

  /// A [`Symbol`](https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Symbol)
  /// that can be used to declare custom `inspect` functions.
  external dynamic get custom;

  /// An object providing the default options used by this function.
  external InspectOptions get defaultOptions;

  /// An object associating style names to their color.
  external dynamic get styles;

  /// Returns a string representation of [object] that is primarily useful for debugging.
  /// Additional [options] may be passed that alter certain aspects of the formatted string.
  external String call(object, [InspectOptions options]);
}

/// Options used to alter certain aspects of a formatted string.
@JS()
@anonymous
class InspectOptions {

  /// Creates a new options object.
  external factory InspectOptions({
    int breakLength = 60,
    bool colors = false,
    bool customInspect = true,
    int depth = 2,
    int maxArrayLength = 100,
    bool showHidden = false,
    bool showProxy = false
  });

  /// The length at which an object's keys are split across multiple lines.
  /// Set to [double.INFINITY] to format an object as a single line.
  external int get breakLength;
  external set breakLength(int value);

  /// Value indicating whether the output will be styled with ANSI color codes.
  external bool get colors;
  external set colors(bool value);

  /// Value indicating whether custom inspection functions exported on the object being inspected will be called.
  external bool get customInspect;
  external set customInspect(bool value);

  /// Specifies the number of times to recurse while formatting the object.
  /// To make it recurse indefinitely pass `null`.
  external int get depth;
  external set depth(int value);

  /// Specifies the maximum number of list elements to include when formatting.
  /// Set to `null` to show all list elements. Set to `0` or negative to show no list elements.
  external int get maxArrayLength;
  external set maxArrayLength(int value);

  /// Value indicating whether the object's non-enumerable symbols and properties will be included in the formatted result.
  external bool get showHidden;
  external set showHidden(bool value);

  /// Value indicating whether objects and functions that are proxy objects will be introspected to show their target and handler objects.
  external bool get showProxy;
  external set showProxy(bool value);
}
