part of nodejs_interop.node;

/// Options used to customize the text decoding.
@JS()
@anonymous
class DecodeOptions {

  /// Creates a new options object.
  external factory DecodeOptions({bool stream = false});

  /// Value indicating whether additional chunks of data are expected.
  external bool get stream;
  external set stream(bool value);
}

/// Takes a stream of bytes as input and emits a stream of code points.
@JS(r'$util.TextDecoder')
class TextDecoder {

  /// Creates a new text decoder.
  external TextDecoder([String encoding, TextDecoderOptions options]);

  /// The encoding supported by this instance.
  external String get encoding;

  /// Value indicating whether decoding failures are fatal.
  external bool get fatal;

  /// Value indicating whether to include the byte order mark in the decoded result.
  external bool get ignoreBOM;

  /// Decodes the [input] array and returns a string containing the decoded bytes.
  external String decode([TypedData input, DecodeOptions options]);
}

/// Options passed to text decoders.
@JS()
@anonymous
class TextDecoderOptions {

  /// Creates a new options object.
  external factory TextDecoderOptions({bool fatal = false, bool ignoreBOM = false});

  /// Value indicating whether decoding failures are fatal.
  external bool get fatal;
  external set fatal(bool value);

  /// Value indicating whether to include the byte order mark in the decoded result.
  /// This option is only used when encoding is `"utf-8"`, `"utf-16be"` or `"utf-16le"`.
  external bool get ignoreBOM;
  external set ignoreBOM(bool value);
}
