part of nodejs_interop.node;

/// Represents a precompiled script that can be executed in specific sandboxes (or "contexts").
@JS(r'$vm.Script')
class Script {

  /// Creates a new script from the specified [code].
  external Script(String code, [ScriptOptions options]);

  /// Runs the compiled code contained by this script within the context of the [contextifiedSandbox] and returns the result.
  /// Running code does not have access to the local scope.
  external dynamic runInContext(contextifiedSandbox, [ContextOptions options]);

  /// Contextifies the given `sandbox`, runs the compiled code contained by this script within the created sandbox, and returns the result.
  /// Running code does not have access to the local scope.
  external dynamic runInNewContext([sandboxOrOptions, ContextOptions options]);

  /// Runs the compiled code contained by this script within the context of the current `global` object and returns the result.
  /// Running code does not have access to local scope, but does have access to the current `global` object.
  external dynamic runInThisContext([ContextOptions options]);
}

/// Options used to customize the compilation of JavaScript code.
@JS()
@anonymous
class ScriptOptions extends ContextOptions {

  /// Creates a new options object.
  external factory ScriptOptions({
    Uint8List cachedData,
    int columnOffset,
    bool displayErrors,
    String filename,
    int lineOffset,
    bool produceCachedData,
    int timeout
  });

  /// Provides an optional [Uint8List] with V8's code cache data for the supplied source.
  external Uint8List get cachedData;
  external set cachedData(Uint8List value);

  /// Value indicating whether V8 will attempt to produce code cache data if no [cachedData] is present.
  external bool get produceCachedData;
  external set produceCachedData(bool value);
}
