/// Support for interoperating with [`make-dir`](https://www.npmjs.com/package/make-dir) package.
@JS()
library nodejs_interop.npm.make_dir;

import 'package:js/js.dart';
import 'package:nodejs_interop/js.dart';
import 'package:nodejs_interop/node.dart';

/// The [`make-dir`](https://www.npmjs.com/package/make-dir) module.
@JS()
abstract class MakeDirModule implements Function {

  /// Creates the directory specified by [path], and its parents if needed.
  /// Returns a [Promise] that resolves with the path to the created directory.
  external Promise<String> call(String path, [MakeDirOptions options]);

  /// Synchronously creates the directory specified by [path], and its parents if needed.
  /// Returns the path to the created directory.
  external String sync(String path, [MakeDirOptions options]);
}

/// Options used to customize the directory creation.
@JS()
@anonymous
class MakeDirOptions {

  /// Creates a new options object.
  external factory MakeDirOptions({FSModule fs, int mode});

  /// Use a custom `fs` implementation.
  external FSModule get fs;
  external set fs(FSModule value);

  /// The directory permissions. Defaults to `0x1FF & (~process.umask())`.
  external int get mode;
  external set mode(int value);
}
