/// Support for interoperating with [`supports-color`](https://www.npmjs.com/package/supports-color) package.
@JS()
library nodejs_interop.npm.supports_color;

import 'package:js/js.dart';

/// Defines the level of support for color of a stream.
@JS()
abstract class ColorSupportLevel {

  /// The support level as an integer.
  external int get level;

  /// Value indicating whether the stream has basic color support (i.e. 16 colors).
  external bool get hasBasic;

  /// Value indicating whether the stream has 256-color support.
  external bool get has256;

  /// Value indicating whether the stream has truecolor support (i.e. 16 million colors).
  external bool get has16m;
}

/// The [`supports-color`](https://www.npmjs.com/package/supports-color) module.
@JS()
abstract class SupportsColorModule {

  /// The level of support for color of the `stderr` stream.
  ///
  /// Returns `false` if `stderr` does not support color.
  /// Returns a [ColorSupportLevel] object if `stderr` supports color.
  external dynamic get stderr;

  /// The level of support for color of the `stdout` stream.
  ///
  /// Returns `false` if `stdout` does not support color.
  /// Returns a [ColorSupportLevel] object if `stdout` supports color.
  external dynamic get stdout;
}
