import 'package:nodejs_interop/io.dart';
import 'package:test/test.dart';

/// Tests the features of the [FileStat] class.
void main() => group('FileStat', () {
  final matchesPerms = anyOf(equals(int.parse('664', radix: 8)), equals(int.parse('644', radix: 8)));

  group('.modeString()', () {
    test('should return a string like "rw-r--r--" for an existing entity', () async {
      var stat = await FileStat.stat('test/fixtures/file.txt');
      expect(stat.modeString(), anyOf(equals('rw-rw-r--'), equals('rw-r--r--')));
    });

    test('should return "---------" if an error occurred', () async {
      var stat = await FileStat.stat('not_found.dat');
      expect(stat.modeString(), equals('---------'));
    });
  });

  group('.stat()', () {
    test('should return a `FileStat` instance for an existing entity', () async {
      var stat = await FileStat.stat('test/fixtures/file.txt');
      expect(stat.mode & 0xFFF, matchesPerms);
      expect(stat.size, equals(0));
      expect(stat.type, equals(FileSystemEntityType.FILE));
    });

    test('should return an instance with a `FileSystemEntityType.NOT_FOUND` type if an error occurred', () async {
      var stat = await FileStat.stat('not_found.dat');
      expect(stat.type, equals(FileSystemEntityType.NOT_FOUND));
    });
  });

  group('.statSync()', () {
    test('should return a `FileStat` instance for an existing entity', () {
      var stat = FileStat.statSync('test/fixtures/file.txt');
      expect(stat.mode & 0xFFF, matchesPerms);
      expect(stat.size, equals(0));
      expect(stat.type, equals(FileSystemEntityType.FILE));
    });

    test('should return an instance with a `FileSystemEntityType.NOT_FOUND` type if an error occurred', () {
      var stat = FileStat.statSync('not_found.dat');
      expect(stat.type, equals(FileSystemEntityType.NOT_FOUND));
    });
  });
});
