import 'package:file/testing.dart';
import 'package:nodejs_interop/io.dart';
import 'package:path/path.dart' as path;
import 'package:test/test.dart';

/// Tests the features of the [File] class.
void main() => group('File', () {
  group('.absolute', () {
    test('should return a `File` object with an absolute path', () {
      expect(new File('/foo.bar').absolute, hasPath('/foo.bar'));
      expect(new File('test/fixtures/file.txt').absolute, hasPath(path.absolute('test/fixtures/file.txt')));
    });
  });

  group('.uri', () {
    test('should return a `Uri` with a `file` scheme', () {
      expect(new File('/foo.bar').uri, equals(Uri.parse('file:///foo.bar')));
      expect(new File('/test/fixtures/file.txt').uri, equals(Uri.parse('file:///test/fixtures/file.txt')));
    });
  });

  group('.copy()', () {
    test('should create a copy of the specified file', () async {
      var file = new File('var/test/file/file.copy/foo');
      expect(file, isNot(exists));
      await file.parent.create(recursive: true);

      await new File('test/fixtures/file.txt').copy(file.path);
      expect(file, exists);
    });
  });

  group('.copySync()', () {
    test('should create a copy of the specified file', () {
      var file = new File('var/test/file/file.copySync/foo');
      expect(file, isNot(exists));
      file.parent.createSync(recursive: true);

      new File('test/fixtures/file.txt').copySync(file.path);
      expect(file, exists);
    });
  });

  group('.create()', () {
    test('should recursively create the specified file', () async {
      var file = new File('var/test/file/file.create/foo');
      expect(file, isNot(exists));

      await file.create(recursive: true);
      expect(file, exists);

      var child = file.parent.childFile('bar');
      await child.create();
      expect(child, exists);
    });
  });

  group('.createSync()', () {
    test('should recursively create the specified file', () {
      var file = new File('var/test/file/file.createSync/foo');
      expect(file, isNot(exists));

      file.createSync(recursive: true);
      expect(file, exists);
      expect(file.parent.childFile('bar')..createSync(), exists);
    });
  });

  /*
  group('.delete()', () {
    test('should TODO', () async {
      // TODO
    });
  });

  group('.deleteSync()', () {
    test('should TODO', () {
      // TODO
    });
  });

  group('.rename()', () {
    test('should TODO', () async {
      // TODO
    });
  });

  group('.renameSync()', () {
    test('should TODO', () {
      // TODO
    });
  });
  */
});
