import 'package:file/testing.dart';
import 'package:nodejs_interop/io.dart';
import 'package:path/path.dart' as path;
import 'package:test/test.dart';

/// Tests the features of the [Link] class.
void main() => group('Link', () {
  group('.absolute', () {
    test('should return a `Link` object with an absolute path', () {
      expect(new Link('/foo.bar').absolute, hasPath('/foo.bar'));
      expect(new Link('test/fixtures/link.lnk').absolute, hasPath(path.absolute('test/fixtures/link.lnk')));
    });
  });

  group('.uri', () {
    test('should return a `Uri` with a `file` scheme', () {
      expect(new Link('/foo.bar').uri, equals(Uri.parse('file:///foo.bar')));
      expect(new Link('/test/fixtures/link.lnk').uri, equals(Uri.parse('file:///test/fixtures/link.lnk')));
    });
  });

  group('.create()', () {
    test('should recursively create the specified link', () async {
      var link = new Link('var/test/file/link.create/foo');
      expect(link, isNot(exists));

      await link.create('/etc/environment', recursive: true);
      expect(link, exists);

      var child = link.parent.childLink('bar');
      await child.create('/etc/environment');
      expect(child, exists);
    });
  });

  group('.createSync()', () {
    test('should recursively create the specified link', () {
      var link = new Link('var/test/file/link.createSync/foo');
      expect(link, isNot(exists));

      link.createSync('/etc/environment', recursive: true);
      expect(link, exists);
      expect(link.parent.childLink('bar')..createSync('/etc/environment'), exists);
    });
  });

  /*
  group('.delete()', () {
    test('should TODO', () async {
      // TODO
    });
  });

  group('.deleteSync()', () {
    test('should TODO', () {
      // TODO
    });
  });

  group('.rename()', () {
    test('should TODO', () async {
      // TODO
    });
  });

  group('.renameSync()', () {
    test('should TODO', () {
      // TODO
    });
  });

  group('.target()', () {
    test('should TODO', () async {
      // TODO
    });
  });

  group('.targetSync()', () {
    test('should TODO', () {
      // TODO
    });
  });

  group('.update()', () {
    test('should TODO', () async {
      // TODO
    });
  });

  group('.updateSync()', () {
    test('should TODO', () {
      // TODO
    });
  });
  */
}, testOn: 'posix');
