import 'package:nodejs_interop/io.dart';
import 'package:nodejs_interop/node.dart';
import 'package:test/test.dart';

/// Tests the features of the `Process` class and functions.
void main() {
  group('exitCode', () {
    test('should get or set the process exit code', () {
      expect(exitCode, equals(0));
      exitCode = 666;
      expect(exitCode, equals(666));
      expect(process.exitCode, equals(666));
      exitCode = 0;
      expect(process.exitCode, equals(0));
    });
  });

  group('pid', () {
    test('should return the PID of the current process', () {
      expect(pid, isPositive);
    });
  });

  group('sleep()', () {
    test('should synchronously wait for the given duration', () {
      var stopwatch = new Stopwatch()..start();
      sleep(const Duration(seconds: 1));
      stopwatch.stop();
      expect(stopwatch.elapsed.inSeconds, equals(1));

      stopwatch..reset()..start();
      sleep(const Duration(seconds: 2));
      stopwatch.stop();
      expect(stopwatch.elapsed.inSeconds, equals(2));
    });
  });
}
