import 'dart:convert';
import 'package:nodejs_interop/node.dart';
import 'package:test/test.dart';

/// Tests the features of the [BufferModule] class.
void main() => group('BufferModule', () {
  final BufferModule buffer = require('buffer');

  group('.transcode()', () {
    test('should re-encode the given source buffer', () {
      var output = buffer.transcode(Buffer.from('Cédric'), BufferEncoding.utf8, BufferEncoding.latin1);
      expect(output, orderedEquals(LATIN1.encode('Cédric')));
    });

    test('should throw an error if an encoding is invalid', () {
      var input = Buffer.from('Zm9vQmFy', BufferEncoding.base64);
      expect(() => buffer.transcode(input, BufferEncoding.base64, BufferEncoding.utf8), throwsException);
    });

    test('should use substitution characters if a byte sequence cannot be represented', () {
      var output = buffer.transcode(Buffer.from('Cédric'), BufferEncoding.utf8, BufferEncoding.ascii);
      expect(output, orderedEquals(ASCII.encode('C?dric')));
    });
  });
});
