import 'package:nodejs_interop/node.dart';
import 'package:test/test.dart';

/// Tests the features of the [DnsModule] class.
void main() => group('DnsModule', () {
  final DnsModule dns = require('dns');
  final NetModule net = require('net');

  group('.lookup()', () {
    test('should resolve the hostname into the first found IP address', () async {
      var result = await fn1cb2ToFuture<String, int>(dns.lookup, 'belin.io');
      expect(net.isIPv4(result.item1), isTrue);
      expect(result.item2, equals(4));
    });

    test('should resolve the hostname into an IPv6 address', () async {
      var result = await fn2cb2ToFuture<String, int>(dns.lookup, 'google.com', 6);
      expect(net.isIPv6(result.item1), isTrue);
      expect(result.item2, equals(6));
    });

    test('should resolve the hostname into a list of IP addresses', () async {
      var addresses = await fn2cb1ToFuture<List<IPAddress>>(dns.lookup, 'belin.io', new LookupOptions(all: true));
      expect(addresses, hasLength(1));
      expect(net.isIPv4(addresses.first.address), isTrue);
      expect(addresses.first.family, equals(4));
    });
  });

  group('.lookupService()', () {
    test('should resolve the given address and port into a `hostname` and `service`', () async {
      var result = await fn2cb2ToFuture<String, String>(dns.lookupService, '163.172.221.182', 80);
      expect(result.item1, anyOf(equals('ns.belin.io'), equals('sd-100506'), equals('sd-100506.dedibox.fr')));
      expect(result.item2, equals('http'));
    });
  });
});
