import 'package:nodejs_interop/node.dart';
import 'package:test/test.dart';

/// Tests the features of the [NetModule] class.
void main() => group('NetModule', () {
  final NetModule net = require('net');

  var ipv4Address = '163.172.221.182';
  var ipv4MappedIpv6Address = '::ffff:a3ac:ddb6';
  var ipv6Address = '2a00:1450:4007:811::200e';

  group('.isIP()', () {
    test('should return `4` for a valid IPv4 address', () {
      expect(net.isIP(ipv4Address), equals(4));
    });

    test('should return `6` for a valid IPv6 address', () {
      expect(net.isIP(ipv4MappedIpv6Address), equals(6));
      expect(net.isIP(ipv6Address), equals(6));
    });

    test('should return `0` for an invalid address', () {
      expect(net.isIP('foo.bar'), equals(0));
    });
  });

  group('.isIPv4()', () {
    test('should return `true` for a valid IPv4 address', () {
      expect(net.isIPv4(ipv4Address), isTrue);
    });

    test('should return `false` for an IPv6 or invalid address', () {
      expect(net.isIPv4(ipv4MappedIpv6Address), isFalse);
      expect(net.isIPv4(ipv6Address), isFalse);
      expect(net.isIPv4('foo.bar'), isFalse);
    });
  });

  group('.isIPv6()', () {
    test('should return `true` for a valid IPv6 address', () {
      expect(net.isIPv6(ipv4MappedIpv6Address), isTrue);
      expect(net.isIPv6(ipv6Address), isTrue);
    });

    test('should return `false` for an IPv4 or invalid address', () {
      expect(net.isIPv6(ipv4Address), isFalse);
      expect(net.isIPv6('foo.bar'), isFalse);
    });
  });
});
