import 'package:nodejs_interop/js.dart';
import 'package:nodejs_interop/node.dart';
import 'package:test/test.dart';

/// Tests the features of the [QueryStringModule] class.
void main() => group('QueryStringModule', () {
  final QueryStringModule querystring = require('querystring');

  group('.escape()', () {
    test('should perform URL percent-encoding on the given string', () {
      expect(querystring.escape(r';,/?:@&=+$'), equals('%3B%2C%2F%3F%3A%40%26%3D%2B%24'));
      expect(querystring.escape("-_.!~*'()"), equals("-_.!~*'()"));
      expect(querystring.escape('#'), equals('%23'));
      expect(querystring.escape('ABC abc 123'), equals('ABC%20abc%20123'));
    });
  });

  group('.parse()', () {
    test('should parse a URL query string into a collection of key and value pairs', () {
      var result = querystring.parse('foo=bar&abc=xyz&abc=123');
      expect(getProperty(result, 'foo'), equals('bar'));
      expect(getProperty(result, 'abc'), orderedEquals(<String>['xyz', '123']));
    });
  });

  group('.stringify()', () {
    test('should produce a URL query string from the given object', () {
      Object object = jsify({'foo': 'bar', 'baz': ['qux', 'quux'], 'corge': ''});
      expect(querystring.stringify(object), equals('foo=bar&baz=qux&baz=quux&corge='));
    });

    test('should handle different `separator` and `equal` arguments', () {
      Object object = jsify({'foo': 'bar', 'baz': 'qux'});
      expect(querystring.stringify(object, ';', ':'), equals('foo:bar;baz:qux'));
    });
  });

  group('.unescape()', () {
    test('should perform decoding of URL percent-encoded characters on the given string', () {
      expect(querystring.unescape('%3B%2C%2F%3F%3A%40%26%3D%2B%24'), equals(r';,/?:@&=+$'));
      expect(querystring.unescape("-_.!~*'()"), equals("-_.!~*'()"));
      expect(querystring.unescape('%23'), equals('#'));
      expect(querystring.unescape('ABC%20abc%20123'), equals('ABC abc 123'));
    });
  });
});
