import 'dart:async';
import 'package:grinder/grinder.dart';

/// Starts the build system.
Future main(List<String> args) => grind(args);

/// Deletes all generated files and reset any saved state.
@Task('Delete the generated files')
void clean() {
  defaultClean();
  delete(getDir('var/test'));
}

/// Checks the package dependencies.
@Task('Check the package dependencies')
void deps() {
  run('node_modules/.bin/david');
  run('node_modules/.bin/nsp', arguments: const ['check']);
}

/// Builds the documentation.
@Task('Build the documentation')
void doc() {
  delete(getDir('doc/api'));
  DartDoc.doc();
}

/// Fixes the coding standards issues.
@Task('Fix the coding issues')
void fix() => DartFmt.format(existingSourceDirs);

/// Performs static analysis of source code.
@Task('Perform the static analysis')
void lint() => Analyzer.analyze(existingSourceDirs);

/// Runs all the test suites.
@DefaultTask('Run the tests')
void test() {
  delete(getDir('var/test'));
  new TestRunner().test(concurrency: 1, platformSelector: 'node');
}
